# Copyleft (ɔ) 2021 Mauko Quiroga <mauko@pm.me>
#
# Licensed under the EUPL-1.2-or-later
# For details: https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

"""Module containing the cli tasks.

.. versionadded:: 1.0.0

"""

from ._tasks import Tasks  # noqa: F401
