# Copyleft (ɔ) 2021 Mauko Quiroga <mauko@pm.me>
#
# Licensed under the EUPL-1.2-or-later
# For details: https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

"""Commons utilities.

.. versionadded:: 1.0.0

"""

from ._arrays import add, fill, pop, pre, rep  # noqa: F401
from ._functions import (  # noqa: F401
    chain,
    compact,
    cons,
    dcons,
    dmap,
    first,
    flatten,
    partial,
    )
