# Copyleft (ɔ) 2021 Mauko Quiroga <mauko@pm.me>
#
# Licensed under the EUPL-1.2-or-later
# For details: https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

"""Domain models of :mod:`mantic`.

.. versionadded:: 1.0.0

"""

from ._argument import Argument  # noqa: F401
from ._exit import Exit  # noqa: F401
from ._signature import Signature  # noqa: F401
from ._suffix import Suffix  # noqa: F401
from ._to_value import to_value  # noqa: F401
from ._version import VersionInt, VersionStr  # noqa: F401
